@@ -21,11 +21,13 @@
  */
 
 #include <config.h>
-#include <stdio.h>
 #include <string.h>
+#include <stdio.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <time.h>
+#include <locale.h>
+#include <ctype.h>
 #include <gtk/gtk.h>
 #include <libgnome/libgnome.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -33,6 +35,7 @@
 #include "catalog-web-exporter.h"
 #include "comments.h"
 #include "file-utils.h"
+#include "jpegutils/jpeg-data.h"
 #include "gth-exif-utils.h"
 #include "dlg-file-utils.h"
 #include "gthumb-init.h"
@@ -118,7 +121,7 @@ image_data_new (const char *filename)
 	idata = g_new (ImageData, 1);
 
 	cdata = comments_load_comment (filename);
-	idata->comment = comments_get_comment_as_string (cdata, "&nbsp;<BR>", "&nbsp;<BR>");
+	idata->comment = comments_get_comment_as_xml_string (cdata, "&nbsp;<br />", "&nbsp;<br />");
 	if (cdata != NULL)
 		comment_data_free (cdata);
 
@@ -826,17 +829,45 @@ get_page_idx_from_image_idx (CatalogWebExporter *ce,
 }
 
 
+static gboolean
+line_is_void (const char *line)
+{
+	const char *scan;
+
+	if (line == NULL)
+		return TRUE;
+
+	for (scan = line; *scan != '\0'; scan++)
+		if ((*scan != ' ')
+		    && (*scan != '\t')
+		    && (*scan != '\n'))
+			return FALSE;
+
+	return TRUE;
+}
+
+
 static void
 write_line (const char *line, FILE *fout) 
 {
-	if (line == NULL)
-		return;
-	if (*line == 0)
+	if (line_is_void (line))
 		return;
+
 	fwrite (line, sizeof (char), strlen (line), fout);
 }
 
 
+static void
+write_markup_escape_line (const char *line, FILE *fout) 
+{
+	char *e_line;
+
+	e_line = _g_escape_text_for_html (line, -1);
+	write_line (e_line, fout);
+	g_free (e_line);
+}
+
+
 static void
 write_locale_line (const char *line, FILE *fout) 
 {
@@ -853,6 +884,17 @@ write_locale_line (const char *line, FILE *fout)
 }
 
 
+static void
+write_markup_escape_locale_line (const char *line, FILE *fout) 
+{
+	char *e_line;
+	
+	e_line = _g_escape_text_for_html (line, -1);
+	write_locale_line (e_line, fout);
+	g_free (e_line);
+}
+
+
 static char *
 get_thumbnail_filename (CatalogWebExporter *ce,
 			ImageData          *idata,
@@ -918,6 +960,52 @@ get_current_date (void)
 }
 
 
+static int
+is_alpha_string (char   *s, 
+		 size_t  maxlen)
+{
+	if (s == NULL)
+		return 0;
+	
+	while ((maxlen > 0) && (*s != '\0') && isalpha (*s)) {
+		maxlen--;
+		s++;
+	}
+	
+	return ((maxlen == 0) || (*s == '\0'));
+}
+
+
+static char*
+get_current_language (void)
+{
+	char   *language = NULL;
+	char   *tmp_locale;
+	char   *locale;
+	char   *underline;
+	size_t  len;
+
+	tmp_locale = setlocale (LC_ALL, NULL);
+	if (tmp_locale == NULL)
+		return NULL;
+	locale = g_strdup (tmp_locale);
+
+	/* FIXME: complete LC_ALL -> RFC 3066 */
+	
+	underline = strchr (locale, '_');
+	if (underline != NULL)
+		*underline = '\0';
+
+	len = strlen (locale);
+	if (((len == 2) || (len == 3)) && is_alpha_string (locale, len))
+		language = g_locale_to_utf8 (locale, -1, 0, 0, 0);
+		
+	g_free (locale);
+
+	return language;
+}
+
+
 static char *
 get_hf_text (const char *utf8_text)
 {
@@ -981,7 +1069,7 @@ gth_parsed_doc_print (GList              *document,
 		ImageData  *idata;
 		char       *line = NULL;
 		char       *image_src, *image_src_relative;
-		char       *escaped_path;
+		char       *escaped_path, *e_escaped_path;
 		int         idx;
 		int         image_width;
 		int         image_height;
@@ -997,12 +1085,17 @@ gth_parsed_doc_print (GList              *document,
 		switch (tag->type) {
 		case GTH_TAG_HEADER:
 			line = get_hf_text (ce->header);
-			write_line (line, fout);
+			write_markup_escape_line (line, fout);
 			break;
 
 		case GTH_TAG_FOOTER:
 			line = get_hf_text (ce->footer);
-			write_line (line, fout);
+			write_markup_escape_line (line, fout);
+			break;
+
+		case GTH_TAG_LANGUAGE:
+			line = get_current_language ();
+			write_markup_escape_line (line, fout);
 			break;
 
 		case GTH_TAG_IMAGE:
@@ -1040,10 +1133,11 @@ gth_parsed_doc_print (GList              *document,
 			image_src_relative = get_path_relative_to_dir (image_src, 
 								       ce->location);
 			escaped_path = escape_uri (image_src_relative);
+			e_escaped_path = _g_escape_text_for_html (escaped_path, -1);
 
-			line = g_strdup_printf ("<img src=\"%s\" alt=\"%s\" width=\"%d\" height=\"%d\"%s/>",
-						escaped_path,
-						escaped_path,
+			line = g_strdup_printf ("<img src=\"%s\" alt=\"%s\" width=\"%d\" height=\"%d\"%s />",
+						e_escaped_path,
+						e_escaped_path,
 						image_width,
 						image_height,
 						class_attr);
@@ -1051,6 +1145,7 @@ gth_parsed_doc_print (GList              *document,
 			g_free (image_src);
 			g_free (image_src_relative);
 			g_free (escaped_path);
+			g_free (e_escaped_path);
 			write_line (line, fout);
 			break;
 
@@ -1064,7 +1159,7 @@ gth_parsed_doc_print (GList              *document,
 			g_free (line);
 			line = escaped_path;
 
-			write_line (line, fout);
+			write_markup_escape_line (line, fout);
 			break;
 
 		case GTH_TAG_IMAGE_IDX:
@@ -1106,12 +1201,12 @@ gth_parsed_doc_print (GList              *document,
 			}
 
 			if  (gth_tag_get_var (ce, tag, "utf8") != 0) 
-				write_locale_line (line, fout);
+				write_markup_escape_locale_line (line, fout);
 			else {
 				escaped_path = escape_uri (line);
 				g_free (line);
 				line = escaped_path;
-				write_line (line, fout);
+				write_markup_escape_line (line, fout);
 			}
 
 			break;
@@ -1131,12 +1226,12 @@ gth_parsed_doc_print (GList              *document,
 			g_free (filename);
 
 			if  (gth_tag_get_var (ce, tag, "utf8") != 0) 
-				write_locale_line (line, fout);
+				write_markup_escape_locale_line (line, fout);
 			else {
 				escaped_path = escape_uri (line);
 				g_free (line);
 				line = escaped_path;
-				write_line (line, fout);
+				write_markup_escape_line (line, fout);
 			}
 	
 			break;
@@ -1145,7 +1240,7 @@ gth_parsed_doc_print (GList              *document,
 			idx = get_image_idx (tag, ce);
 			idata = g_list_nth (ce->file_list, idx)->data;
 			line = gnome_vfs_format_file_size_for_display (idata->file_size);
-			write_line (line, fout);
+			write_markup_escape_line (line, fout);
 			break;
 
 		case GTH_TAG_COMMENT:
@@ -1186,7 +1281,7 @@ gth_parsed_doc_print (GList              *document,
 						    zero_padded (idx + 1),
 						    ".html",
 						    NULL);
-			write_line (line, fout);
+			write_markup_escape_line (line, fout);
 			break;
 
 		case GTH_TAG_PAGE_IDX:
@@ -1204,7 +1299,10 @@ gth_parsed_doc_print (GList              *document,
 				break;
 
 			for (r = 0; r < ce->page_rows; r++) {
-				write_line ("  <tr class=\"tr_index\">\n", fout);
+				if (ce->image < ce->n_images)
+					write_line ("  <tr class=\"tr_index\">\n", fout);
+				else
+					write_line ("  <tr class=\"tr_empty_index\">\n", fout);
 				for (c = 0; c < ce->page_cols; c++) {
 					if (ce->image < ce->n_images) {
 						write_line ("    <td class=\"td_index\">\n", fout);
@@ -1215,10 +1313,7 @@ gth_parsed_doc_print (GList              *document,
 						write_line ("    </td>\n", fout);
 						ce->image++;
 					} else {
-						char *line;
-						line = g_strdup_printf ("    <td width=%d height=%d>\n", ce->thumb_width, ce->thumb_height);
-						write_line (line, fout);
-						g_free (line);
+						write_line ("    <td class=\"td_empty_index\">\n", fout);
 						write_line ("    &nbsp;\n", fout);
 						write_line ("    </td>\n", fout);
 					} 
@@ -1229,7 +1324,7 @@ gth_parsed_doc_print (GList              *document,
 
 		case GTH_TAG_DATE:
 			line = get_current_date ();
-			write_line (line, fout);
+			write_markup_escape_line (line, fout);
 			break;
 
 		case GTH_TAG_HTML:
@@ -1242,7 +1337,7 @@ gth_parsed_doc_print (GList              *document,
 #ifdef HAVE_LIBEXIF
 			line = get_exif_tag (idata->src_filename, 
 					     EXIF_TAG_EXPOSURE_TIME);
-			write_line (line, fout);
+			write_markup_escape_line (line, fout);
 #endif /* HAVE_LIBEXIF */
 			break;
 
@@ -1252,7 +1347,7 @@ gth_parsed_doc_print (GList              *document,
 #ifdef HAVE_LIBEXIF
 			line = get_exif_tag (idata->src_filename, 
 					     EXIF_TAG_EXPOSURE_MODE);
-			write_line (line, fout);
+			write_markup_escape_line (line, fout);
 #endif /* HAVE_LIBEXIF */
 			break;
 
@@ -1261,7 +1356,7 @@ gth_parsed_doc_print (GList              *document,
 			idata = g_list_nth (ce->file_list, idx)->data;
 #ifdef HAVE_LIBEXIF
 			line = get_exif_tag (idata->src_filename, EXIF_TAG_FLASH);
-			write_line (line, fout);
+			write_markup_escape_line (line, fout);
 #endif /* HAVE_LIBEXIF */
 			break;
 
@@ -1271,7 +1366,7 @@ gth_parsed_doc_print (GList              *document,
 #ifdef HAVE_LIBEXIF
 			line = get_exif_tag (idata->src_filename, 
 					     EXIF_TAG_SHUTTER_SPEED_VALUE);
-			write_line (line, fout);
+			write_markup_escape_line (line, fout);
 #endif /* HAVE_LIBEXIF */
 			break;
 
@@ -1280,7 +1375,7 @@ gth_parsed_doc_print (GList              *document,
 			idata = g_list_nth (ce->file_list, idx)->data;
 #ifdef HAVE_LIBEXIF
 			line = get_exif_aperture_value (idata->src_filename);
-			write_line (line, fout);
+			write_markup_escape_line (line, fout);
 #endif /* HAVE_LIBEXIF */
 			break;
 
@@ -1290,7 +1385,7 @@ gth_parsed_doc_print (GList              *document,
 #ifdef HAVE_LIBEXIF
 			line = get_exif_tag (idata->src_filename, 
 					     EXIF_TAG_FOCAL_LENGTH);
-			write_line (line, fout);
+			write_markup_escape_line (line, fout);
 #endif /* HAVE_LIBEXIF */
 			break;
 
@@ -1308,7 +1403,7 @@ gth_parsed_doc_print (GList              *document,
 					tp = localtime (&t);
 					strftime (s, 99, DATE_FORMAT, tp);
 					line = g_locale_to_utf8 (s, -1, 0, 0, 0);
-					write_line (line, fout);
+					write_markup_escape_line (line, fout);
 				} else
 					write_line ("-", fout);
 
@@ -1322,14 +1417,14 @@ gth_parsed_doc_print (GList              *document,
 #ifdef HAVE_LIBEXIF
 			line = get_exif_tag (idata->src_filename, 
 					     EXIF_TAG_MAKE);
-			write_line (line, fout);
+			write_markup_escape_line (line, fout);
 			g_free (line);
 
 			write_line (" ", fout);
 
 			line = get_exif_tag (idata->src_filename, 
 					     EXIF_TAG_MODEL);
-			write_line (line, fout);
+			write_markup_escape_line (line, fout);
 #endif /* HAVE_LIBEXIF */
 			break;
 
@@ -1366,7 +1461,7 @@ gth_parsed_doc_print (GList              *document,
 				if (child->type != GTH_TAG_HTML)
 					break;
 				line = g_strdup (_(child->value.html));
-				write_line (line, fout);
+				write_markup_escape_line (line, fout);
 			} 
 			break;
 
@@ -1513,6 +1608,39 @@ export__save_other_files (CatalogWebExporter *ce)
 }
 
 
+static void
+copy_exif_from_orig (const char *src_filename, 
+		     const char *dest_filename)
+{
+#if HAVE_LIBEXIF
+	JPEGData     *jdata_src, *jdata_dest;
+	ExifData     *edata_src;
+
+	jdata_src = jpeg_data_new_from_file (src_filename);
+	if (jdata_src == NULL)
+		return;
+
+	edata_src = jpeg_data_get_exif_data (jdata_src);
+	if (edata_src == NULL) {
+		jpeg_data_unref (jdata_src);
+		return;
+	}
+
+	jdata_dest = jpeg_data_new_from_file (dest_filename);
+	if (jdata_dest == NULL)
+		return;
+
+	jpeg_data_set_exif_data (jdata_dest, edata_src);
+
+	jpeg_data_save_file (jdata_dest, dest_filename);
+
+	exif_data_unref (edata_src);
+	jpeg_data_unref (jdata_src);
+	jpeg_data_unref (jdata_dest);
+#endif
+}
+
+
 static gboolean
 save_thumbnail_cb (gpointer data)
 {
@@ -1544,9 +1672,10 @@ save_thumbnail_cb (gpointer data)
 			if (_gdk_pixbuf_save (idata->thumb,
 					      filename,
 					      "jpeg",
-					      NULL, NULL))
+					      NULL, NULL)) {
+				copy_exif_from_orig (idata->src_filename, filename);
 				ce->album_files = g_list_prepend (ce->album_files, filename);
-			else
+			} else
 				g_free (filename);
 			
 			g_object_unref (idata->thumb);
@@ -1805,9 +1934,10 @@ save_image_preview_cb (gpointer data)
 			if (_gdk_pixbuf_save (idata->preview,
 					      filename,
 					      "jpeg",
-					      NULL, NULL))
+					      NULL, NULL)) {
+				copy_exif_from_orig (idata->src_filename, filename);
 				ce->album_files = g_list_prepend (ce->album_files, filename);
-			else
+			} else
 				g_free (filename);
 		}
 	}
@@ -1842,6 +1972,7 @@ save_resized_image_cb (gpointer data)
 					      filename,
 					      "jpeg",
 					      NULL, NULL)) {
+				copy_exif_from_orig (idata->src_filename, filename);
 				ce->album_files = g_list_prepend (ce->album_files, filename);
 				idata->file_size = get_file_size (filename);
 			} else
@@ -1930,6 +2061,41 @@ export__copy_image (CatalogWebExporter *ce)
 }
 
 
+static GdkPixbuf *
+pixbuf_scale (const GdkPixbuf *src,
+	      int              dest_width,
+	      int              dest_height,
+	      GdkInterpType    interp_type)
+{
+	GdkPixbuf *dest;
+
+	if (! gdk_pixbuf_get_has_alpha (src))
+		return gdk_pixbuf_scale_simple (src, dest_width, dest_height, interp_type);
+	
+	g_return_val_if_fail (src != NULL, NULL);
+	g_return_val_if_fail (dest_width > 0, NULL);
+	g_return_val_if_fail (dest_height > 0, NULL);
+	
+	dest = gdk_pixbuf_new (GDK_COLORSPACE_RGB, gdk_pixbuf_get_has_alpha (src), 8, dest_width, dest_height);
+	if (!dest)
+		return NULL;
+	
+	gdk_pixbuf_composite_color (src,
+				    dest,
+				    0, 0, dest_width, dest_height, 0, 0,
+				    (double) dest_width / gdk_pixbuf_get_width (src),
+				    (double) dest_height / gdk_pixbuf_get_height (src),
+				    interp_type,
+				    255,
+				    0, 0,
+				    200,
+				    0xFFFFFF,
+				    0xFFFFFF);
+
+	return dest;
+}
+	
+
 static void
 image_loader_done (ImageLoader *iloader, 
 		   gpointer     data)
@@ -1950,7 +2116,7 @@ image_loader_done (ImageLoader *iloader,
 		int h = gdk_pixbuf_get_height (pixbuf);
 		if (scale_keepping_ratio (&w, &h, ce->resize_max_width, ce->resize_max_height)) {
 			GdkPixbuf *scaled;
-			scaled = gdk_pixbuf_scale_simple (pixbuf, w, h, GDK_INTERP_BILINEAR);
+			scaled = pixbuf_scale (pixbuf, w, h, GDK_INTERP_BILINEAR);
 			g_object_unref (idata->image);
 			idata->image = scaled;
 		} 
@@ -1972,7 +2138,7 @@ image_loader_done (ImageLoader *iloader,
 					  ce->preview_max_width,
 					  ce->preview_max_height)) {
 			GdkPixbuf *scaled;
-			scaled = gdk_pixbuf_scale_simple (pixbuf, w, h, GDK_INTERP_BILINEAR);
+			scaled = pixbuf_scale (pixbuf, w, h, GDK_INTERP_BILINEAR);
 			g_object_unref (idata->preview);
 			idata->preview = scaled;
 		}
@@ -2003,7 +2169,7 @@ image_loader_done (ImageLoader *iloader,
 					  ce->thumb_width,
 					  ce->thumb_height)) {
 			GdkPixbuf *scaled;
-			scaled = gdk_pixbuf_scale_simple (pixbuf, w, h, GDK_INTERP_BILINEAR);
+			scaled = pixbuf_scale (pixbuf, w, h, GDK_INTERP_BILINEAR);
 			g_object_unref (idata->thumb);
 			idata->thumb = scaled;
 		}
