@@ -26,6 +26,7 @@
 #include <unistd.h>
 #include <gtk/gtk.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
+#include <libgnome/libgnome.h>
 #include <glade/glade.h>
 #include "file-data.h"
 #include "gconf-utils.h"
@@ -69,6 +70,8 @@ typedef struct {
 
 	GList        *file_list;
 	GList        *current_image;
+	GList        *saved_list;
+	GList        *deleted_list;
 
 	int              images;
 	int              image;
@@ -98,6 +101,8 @@ destroy_cb (GtkWidget  *widget,
 		g_object_unref (data->loader);
 
 	file_data_list_free (data->file_list);
+	path_list_free (data->saved_list);
+	path_list_free (data->deleted_list);
 
 	g_strfreev (data->keys);
 	g_strfreev (data->values);	
@@ -133,6 +138,9 @@ load_current_image (DialogData *data)
 	char      *message;
 
 	if (data->stop_convertion || (data->current_image == NULL)) {
+		all_windows_notify_files_created (data->saved_list);
+		all_windows_notify_files_deleted (data->deleted_list);
+		all_windows_add_monitor ();
 		gtk_widget_destroy (data->progress_dialog);
 		gtk_widget_destroy (data->dialog);
 		return;
@@ -202,20 +210,13 @@ save_image_and_remove_original (DialogData *data)
 			       data->keys, 
 			       data->values,
 			       &error)) {
-		GList    *list;
 		FileData *fd = data->current_image->data;
-
-		list = g_list_prepend (NULL, data->new_path);
-		all_windows_notify_files_created (list);
-		g_list_free (list);
+		data->saved_list = g_list_prepend (data->saved_list, g_strdup (data->new_path));
 
 		if (data->remove_original 
 		    && (strcmp (fd->path, data->new_path) != 0)) {
 			unlink (fd->path);
-
-			list = g_list_prepend (NULL, fd->path);
-			all_windows_notify_files_deleted (list);
-			g_list_free (list);
+			data->deleted_list = g_list_prepend (data->deleted_list, g_strdup (fd->path));
 		}
 	} else 
 		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->dialog), &error);
@@ -350,6 +351,8 @@ static void
 ok_cb (GtkWidget  *widget, 
        DialogData *data)
 {
+	all_windows_remove_monitor ();
+
 	data->loader = IMAGE_LOADER (image_loader_new (NULL, FALSE));
 
 	g_signal_connect (G_OBJECT (data->loader),
@@ -411,12 +414,46 @@ stop_convertion_cb (GtkWidget  *widget,
 }
 
 
+/* called when the "help" button is clicked. */
+static void
+help_cb (GtkWidget  *widget, 
+	 DialogData *data)
+{
+	GError *err;
+
+	err = NULL;  
+	gnome_help_display ("gthumb", "gthumb-convert-format", &err);
+	
+	if (err != NULL) {
+		GtkWidget *dialog;
+		
+		dialog = gtk_message_dialog_new (GTK_WINDOW (data->dialog),
+						 0,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE,
+						 _("Could not display help: %s"),
+						 err->message);
+		
+		g_signal_connect (G_OBJECT (dialog), "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+		
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+		
+		gtk_widget_show (dialog);
+		
+		g_error_free (err);
+	}
+}
+
+
 void
 dlg_convert (GThumbWindow *window)
 {
 	DialogData  *data;
 	GtkWidget   *cancel_button;
 	GtkWidget   *ok_button;
+	GtkWidget   *help_button;
 	GtkWidget   *button;
 	GtkWidget   *progress_cancel;
 	GList       *list;
@@ -470,6 +507,7 @@ dlg_convert (GThumbWindow *window)
 
 	ok_button = glade_xml_get_widget (data->gui, "conv_ok_button");
 	cancel_button = glade_xml_get_widget (data->gui, "conv_cancel_button");
+	help_button = glade_xml_get_widget (data->gui, "conv_help_button");
 
 	/**/
 
@@ -523,11 +561,14 @@ dlg_convert (GThumbWindow *window)
 				  "clicked",
 				  G_CALLBACK (gtk_widget_destroy),
 				  G_OBJECT (data->dialog));
-
 	g_signal_connect (G_OBJECT (ok_button), 
 			  "clicked",
 			  G_CALLBACK (ok_cb),
 			  data);
+	g_signal_connect (G_OBJECT (help_button), 
+			  "clicked",
+			  G_CALLBACK (help_cb),
+			  data);
 
 	g_signal_connect (G_OBJECT (progress_cancel), 
 			  "clicked",
